RESULT = "result"
SUCCESS = "success"
ERROR = "error"
MESSAGE = "message"
CODE = "code"

PROPERTIES = 'properties'
MISSED_KEYS = {
    "format": {
        "type": "string"
    }, "$ref": {
        "type": "string",
        "format": "uri"
    }
}
